# LMSAPI/API/Room.py
import os
from decimal import Decimal

from sqlalchemy import create_engine
from sqlalchemy.sql import text

from LMSAPI.api.Models.File import File
from LMSAPI.api.Models.User import User
from flask import current_app, g

from LMSAPI.api.utils.request_utils import RequestUtils


class Rating:
    def __init__(self, cname):
        self.lname = cname

    def has_true_stipend(self):
        conn = current_app.ms.db(self.lname).connect()
        sqlt = "SELECT EXISTS (SELECT 1 FROM rating_schema WHERE stipend = True)"
        stmt = text(sqlt)
        result = conn.execute(stmt).scalar()  # Returns True if exists, False otherwise
        conn.close()
        return result

    def getRatingSchema(self, school_year):
        conn = current_app.ms.db(self.lname).connect()
        sqlt = "SELECT * from rating_schema  where :school_year = ANY(school_year) order by title "
        stmt = text(sqlt)
        stmt = stmt.bindparams(school_year=school_year)
        query = conn.execute(stmt)

        result = [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

        return result

    def getRating(self, schema, grpupid, date, type):
        conn = current_app.ms.db(self.lname).connect()
        sqlt = """SELECT p.mid, p.lastname, p.firstname, p.patronymic, pi.*, v.value, v.points, v.id as personnel_rating_id
                FROM people p
                INNER JOIN groupuser gu ON p.mid = gu.mid
                LEFT OUTER JOIN personnel_indicators pi ON pi.schema = :schema
                LEFT OUTER JOIN personnel_rating_values v ON v.mid = p.mid AND v.indicator = pi.id AND v.period_type = :type AND periodend = :date
                where gu.gid = :grpupid
                order by  p.lastname, p.firstname, p.patronymic, pi.id_group, pi.sorting, pi.title"""

        stmt = text(sqlt)
        stmt = stmt.bindparams(schema=schema)
        stmt = stmt.bindparams(grpupid=grpupid)
        stmt = stmt.bindparams(date=date)
        stmt = stmt.bindparams(type=type)
        query = conn.execute(stmt)

        result = [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

        return result

    def update_rating(self, id, data_dict):
        conn = current_app.ms.db(self.lname).connect()

        sqlt = """
            UPDATE personnel_rating_values 
            SET value = :value
            WHERE id = :id
            """

        stmt = text(sqlt)
        stmt = stmt.bindparams(value=data_dict["value"], id=id)

        try:
            query = conn.execute(stmt)
        except Exception as e:
            return e.args
        return True

    def create_rating(self, data_dict):
        conn = current_app.ms.db(self.lname).connect()

        sqlt = """INSERT INTO personnel_rating_values (mid, indicator, period_type, periodend, value, who_change, when_change)
                           SELECT :mid, :indicator, :period_type, :periodend, :value, :who_change, now() """

        stmt = text(sqlt)
        stmt = stmt.bindparams(
            mid=data_dict["mid"],
            indicator=data_dict["indicator"],
            period_type=data_dict["period_type"],
            periodend=data_dict["periodend"],
            value=data_dict["value"],
            who_change=g.user.mid,
        )

        try:
            query = conn.execute(stmt)
        except Exception as e:
            return e.args
        return True

    def setRating(self, data_dict):
        conn = current_app.ms.db(self.lname).connect()

        sqlt = "SELECT * FROM personnel_rating_values WHERE mid = :mid AND indicator = :indicator AND period_type = :period_type AND periodend = :periodend"
        stmt = text(sqlt)
        stmt = stmt.bindparams(
            mid=data_dict["mid"],
            indicator=data_dict["indicator"],
            period_type=data_dict["period_type"],
            periodend=data_dict["periodend"],
        )
        query = conn.execute(stmt)
        personnel_rating = [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

        if len(personnel_rating) > 0:
            if data_dict["value"] is None or data_dict["value"] == 0:
                self.update_rating_info(
                    personnel_rating[0]["id"],
                    {
                        "title": None,
                        "document_number": None,
                        "document_date": None,
                        "document_date_to": None,
                        "result": None,
                        "info": None,
                        "classification_mark": None,
                    },
                )
                if personnel_rating[0]["link_type"] == 1:
                    os.remove(
                        os.path.join(
                            File(self.lname).get_personnel_rating_values_directory(
                                personnel_rating[0]["id"]
                            ),
                            personnel_rating[0]["link"],
                        )
                    )
                self.update_rating_link(personnel_rating[0]["id"], None, None)
            return self.update_rating(personnel_rating[0]["id"], data_dict)
        else:
            return self.create_rating(data_dict)

    def getRatingCalc(self, schema, grpupid, date, type):
        print(schema, grpupid, date, type)
        conn = current_app.ms.db(self.lname).connect()
        sqlt = """SELECT p.mid, p.lastname, p.firstname, p.patronymic, v.points 
                FROM people p
                INNER JOIN groupuser gu ON p.mid = gu.mid
                LEFT OUTER JOIN personnel_rating_values v ON v.mid = p.mid AND v.indicator = -:schema AND v.period_type = :type AND v.periodend = :date
                where gu.gid = :grpupid
                order by coalesce(v.points, 0) DESC, p.lastname, p.firstname, p.patronymic"""

        stmt = text(sqlt)
        stmt = stmt.bindparams(schema=schema)
        stmt = stmt.bindparams(grpupid=grpupid)
        stmt = stmt.bindparams(date=date)
        stmt = stmt.bindparams(type=type)
        query = conn.execute(stmt)

        result = [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

        return result

    def getRatingCalcUpdated(self, schema, groupid, date, period_type):
        conn = current_app.ms.db(self.lname).connect()
        new_date = None
        period_query = " "

        if period_type:
            period_query = " AND v.period_type = :period_type"
            date_query = " AND v.periodend = :date"
        else:
            new_date = str(int(date) + 1) + "-07-01"
            date = date + "-08-31"
            date_query = " AND v.periodend >= :date AND v.periodend <= :new_date"

        schema_query = " "
        if schema:
            schema_query = " AND v.indicator = :schema "

        sqlt = """SELECT p.mid, p.lastname, p.firstname, p.patronymic, v.points 
                FROM people p
                INNER JOIN groupuser gu ON p.mid = gu.mid
                LEFT OUTER JOIN personnel_rating_values v ON v.mid = p.mid
                    {schema_query}
                    {period_query}
                    {date_query}
                where gu.gid = :groupid
                order by coalesce(v.points, 0) DESC, p.lastname, p.firstname, p.patronymic""".format(
            schema_query=schema_query, period_query=period_query, date_query=date_query
        )

        stmt = text(sqlt)
        stmt = stmt.bindparams(groupid=groupid)
        stmt = stmt.bindparams(date=date)

        if schema:
            stmt = stmt.bindparams(schema=schema)
        if period_type:
            stmt = stmt.bindparams(period_type=period_type)
        else:
            stmt = stmt.bindparams(new_date=new_date)
        query = conn.execute(stmt)

        result = [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

        return result

    def getRatingCalcDetails(self, schema, grpupid, date, type):
        conn = current_app.ms.db(self.lname).connect()
        sqlt = """SELECT p.mid, p.lastname, p.firstname, p.patronymic, vt.points totalpoints, pi.*, vd.points
                FROM people p
                INNER JOIN groupuser gu ON p.mid = gu.mid
                LEFT OUTER JOIN personnel_rating_values vt ON vt.mid = p.mid AND vt.indicator = -:schema AND vt.period_type = :type AND vt.periodend = :date
                LEFT OUTER JOIN personnel_indicators pi ON pi.schema = :schema and (coalesce(pi.id_group, 0) = 0 or pi.top_level = true)
                LEFT OUTER JOIN personnel_rating_values vd ON vd.mid = p.mid AND vd.indicator = pi.id AND vd.period_type = :type AND vd.periodend = :date
                where gu.gid = :grpupid
                order by coalesce(vt.points, 0) DESC, p.lastname, p.firstname, p.patronymic, pi.sorting"""

        stmt = text(sqlt)
        stmt = stmt.bindparams(schema=schema)
        stmt = stmt.bindparams(grpupid=grpupid)
        stmt = stmt.bindparams(date=date)
        stmt = stmt.bindparams(type=type)
        query = conn.execute(stmt)

        result = [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

        return result

    def getDigitalPortrait(self, schema, mid, yid):
        conn = current_app.ms.db(self.lname).connect()
        sqlt = """SELECT p.mid, p.lastname, p.firstname, p.patronymic, pi.*, vd.*
                    FROM people p
                    LEFT OUTER JOIN school_year sy ON sy.xp_key = :yid
                    LEFT OUTER JOIN personnel_indicators pi ON pi.schema = :schema  and (coalesce(pi.id_group, 0) = 0 or pi.top_level = true)
                    LEFT OUTER JOIN personnel_rating_values vd ON vd.mid = p.mid AND vd.indicator = pi.id AND vd.period_type = 1  AND vd.periodend BETWEEN sy.begdate and sy.enddate
                    where p.mid = :mid
                    order by pi.sorting, vd.periodend"""

        stmt = text(sqlt)
        stmt = stmt.bindparams(schema=schema)
        stmt = stmt.bindparams(mid=mid)
        stmt = stmt.bindparams(yid=yid)
        query = conn.execute(stmt)

        result = [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

        return result

    def getRatingEdge(self, schema, type, date, kind, id):
        """
        kind
            1 - группа
            2 - курс
            3 - факультет
            4 - ВУЗ
        """

        conn = current_app.ms.db(self.lname).connect()
        sqlt = """SELECT  pi.*, min(vd.points), max(vd.points)
                    FROM personnel_indicators pi
                    LEFT OUTER JOIN personnel_rating_values vd ON vd.indicator = pi.id AND vd.period_type = :type AND vd.periodend = :date
                    LEFT OUTER JOIN people p ON vd.mid = p.mid
                    LEFT OUTER JOIN groupuser gu ON gu.mid = p.mid and gu.cid = -1
                    LEFT OUTER JOIN groupname g ON gu.gid = g.gid AND g.cid = -1
                    LEFT OUTER JOIN cathedras c ON c.idcathedra = g.idcathedra
                    LEFT OUTER JOIN faculty f ON f.idfaculty = coalesce(g.idfaculty, c.faculty)
                    where  pi.schema = :schema and (coalesce(pi.id_group, 0) = 0 or pi.top_level = true) """

        if kind == 3:
            sqlt = sqlt + " and f.idfaculty = :id"

        if kind == 2:
            sqlt = sqlt + " and g.year = :id"

        if kind == 1:
            sqlt = sqlt + " and g.gid = :id"

        sqlt = sqlt + " group by  pi.id "

        stmt = text(sqlt)
        stmt = stmt.bindparams(schema=schema)
        stmt = stmt.bindparams(type=type)
        stmt = stmt.bindparams(date=date)

        if kind != 4:
            stmt = stmt.bindparams(id=id)

        query = conn.execute(stmt)

        result = [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

        return result

    def getRatingReacal(self, schema, type, date):
        conn = (
            current_app.ms.db(self.lname).connect().execution_options(autocommit=True)
        )

        sqlt = "select refresh_plan_values_calculated(:schema, :type, :date)"
        stmt = text(sqlt)
        stmt = stmt.bindparams(schema=schema)
        stmt = stmt.bindparams(type=type)
        stmt = stmt.bindparams(date=date)
        conn.execute(stmt)

        return True

    def get_rating_info(self, id):
        conn = current_app.ms.db(self.lname).connect()
        sqlt = """
            SELECT * FROM personnel_rating_values WHERE id = :id
            """
        stmt = text(sqlt)
        stmt = stmt.bindparams(id=id)
        query = conn.execute(stmt)
        result = query.fetchone()
        return RequestUtils.convert_decimal_to_float(dict(result))

    def update_rating_info(self, id, data_dict):
        conn = current_app.ms.db(self.lname).connect()

        sqlt = """
            UPDATE personnel_rating_values 
            SET title = :title, document_number = :document_number, document_date = :document_date, 
                document_date_to = :document_date_to, result = :result, info = :info, 
                classification_mark = :classification_mark
            WHERE id = :id
            """

        stmt = text(sqlt)
        stmt = stmt.bindparams(
            id=id,
            title=data_dict["title"],
            document_number=data_dict["document_number"],
            document_date=data_dict["document_date"],
            document_date_to=data_dict["document_date_to"],
            result=data_dict["result"],
            info=data_dict["info"],
            classification_mark=data_dict["classification_mark"],
        )

        try:
            query = conn.execute(stmt)
        except Exception as e:
            return e.args
        return True

    def update_rating_link(self, id, link_type, link):
        conn = current_app.ms.db(self.lname).connect()

        sqlt = """
            UPDATE personnel_rating_values 
            SET link_type = :link_type, link = :link
            WHERE id = :id
            """

        stmt = text(sqlt)
        stmt = stmt.bindparams(
            id=id,
            link_type=link_type,
            link=link,
        )

        try:
            query = conn.execute(stmt)
        except Exception as e:
            return e.args
        return True
