import json
from decimal import Decimal
from typing import Tuple, Union, List, Dict, Any

from flask import current_app
from sqlalchemy import text


class ScholarshipListOptions:
    def __init__(self, cname):
        self.lname = cname

    @staticmethod
    def get_scholarship_list(
        lname: str,
        xp_key: int,
        education_level: int,
        prep_struc_category_id: int,
        prep_levels: Tuple[int] = (),
    ) -> Union[bool, List[Dict[str, Any]]]:
        # 1 - Высшее образование, 2 - Среднее образование
        # prep_struc_category_id: 1 - Курсант, 2 - Слушатель, 3 - Адъюнкт
        # prep_levels - Уровень подготовки из ОПОП
        where = ""
        if prep_levels:
            # Преобразование кортежа в строку для IN фильтра
            placeholders = ", ".join(
                [":prep_level_{i}".format(i=i) for i in range(len(prep_levels))]
            )
            where += " AND dc.preparation_level IN ({placeholders}) ".format(
                placeholders=placeholders
            )

        if prep_struc_category_id == 3:
            record_count = 2
        else:
            record_count = 1

        # Базовый SQL запрос
        sql = """
            SELECT DISTINCT
                dc.mid,
                dc.student_name,
                dc.gid,
                dc.group_name,
                dc.education_level, 
                dc.preparation_level,
                dc.name,
                dc.five_count,
                dc.five_percentage,
                dc.four_count,
                dc.four_percentage,
                dc.three_count,
                dc.three_percentage,
                dc.two_count,
                dc.two_percentage,
                dc.average_grade,
                dc.current_avg_grade,
                dc.record_count,
                dc.record_points,
                dc.record_count_year,
                dc.prep_struc_category_id,
                dc.prep_struc_category,
                dc.third_level_data,
                ey.number yeducationyearid,
                ey.name yeducationyear, 
                jsonb_agg(
                    jsonb_build_object(
                        'scholarship_candidates_id', sc.scholarship_candidates_id,
                        'scholarship_type', sc.scholarship_type
                    )
                ) FILTER (WHERE sc.scholarship_candidates_id IS NOT NULL) AS grants
            FROM degree_candidates7(:xp_key, :education_level, :prep_struc_category_id) dc
            LEFT JOIN scholarship_candidates sc 
                ON dc.mid = sc.mid AND sc.xp_key = :xp_key AND sc.education_level = :education_level
            LEFT JOIN groupname g 
                ON dc.gid = g.gid AND g.cid = -1
            LEFT JOIN educationyears ey 
                ON ey.number = g.year
            LEFT JOIN groupuser_transfer gt
                ON dc.mid = gt.mid
                AND gt.when = (
                    SELECT MAX(gut.when)
                    FROM groupuser_transfer gut
                    WHERE gut.mid = dc.mid
                )
            LEFT JOIN xp_status xs on xs.xp_key = gt.xp_status
            WHERE dc.education_level = :education_level
              AND dc.prep_struc_category_id = :prep_struc_category_id
              AND (xs."Type" != 'Отчисление' AND xs."Type" != 'Выпуск' OR xs."Type" IS NULL)
              AND dc.record_count >= :record_count
              {where}
            GROUP BY 
                dc.mid, dc.student_name, dc.gid, dc.group_name, dc.education_level, 
                dc.preparation_level, dc.name, dc.five_count, dc.five_percentage, 
                dc.four_count, dc.four_percentage, dc.three_count, dc.three_percentage, 
                dc.two_count, dc.two_percentage, dc.average_grade, dc.current_avg_grade, 
                dc.record_count, dc.record_points, dc.record_count_year, 
                dc.prep_struc_category_id, dc.prep_struc_category, dc.third_level_data, 
                ey.number, ey.name
            ORDER BY 
                dc.average_grade DESC NULLS LAST,          -- Сортировка по убыванию среднего балла
                dc.record_points DESC NULLS LAST,            -- Затем по количеству баллов
                dc.current_avg_grade DESC NULLS LAST; 
        """.format(
            where=where
        )

        # Выполнение запроса
        conn = current_app.ms.db(lname).connect()
        stmt = text(sql)
        stmt = stmt.bindparams(
            xp_key=xp_key, education_level=education_level, record_count=record_count
        )

        if prep_struc_category_id != 0:
            stmt = stmt.bindparams(prep_struc_category_id=prep_struc_category_id)

        # Привязываем значения для preparation_level
        for i, level in enumerate(prep_levels):
            stmt = stmt.bindparams(**{"prep_level_{i}".format(i=i): level})

        query = conn.execute(stmt)
        result = [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

        # Convert Decimal to float
        for row in result:
            for key, value in row.items():
                if isinstance(value, Decimal):
                    row[key] = float(value)

        return result

    @staticmethod
    def get_scholarship_list_options(
        lname: str, id, trmid, xp_key, education_level
    ) -> Union[bool, List[Dict[str, Any]]]:
        # Получаем параметры запроса
        filters = []
        params = {}

        # Создаем фильтры
        if id:
            filters.append("id = :id")
            params["id"] = id
        if trmid:
            filters.append("trmid = :trmid")
            params["trmid"] = trmid
        if xp_key:
            filters.append("xp_key = :xp_key")
            params["xp_key"] = xp_key
        if education_level:
            filters.append("education_level = :education_level")
            params["education_level"] = education_level

        # Базовый SQL запрос
        sql = """
            SELECT slo.*, t.title
            FROM scholarship_list_options slo
            JOIN Terms t ON t.trmid = slo.trmid 
            WHERE 1=1
            """

        # Если есть фильтры, добавляем их в запрос
        if filters:
            sql += " AND " + " AND ".join(filters)

        sql += " ORDER BY id"

        # Выполнение запроса
        conn = current_app.ms.db(lname).connect()
        query = conn.execute(text(sql), params)
        res = query.fetchall()

        if not res:
            return False
        return [dict(row) for row in res]

    @staticmethod
    def get_education_level_and_prep_struc_category_id_option(
        lname: str, xp_key
    ) -> Union[bool, List[Dict[str, Any]]]:
        # Базовый SQL запрос
        sql = """
                SELECT DISTINCT prep_struc_category_id, education_level
                FROM scholarship_list_options
                WHERE xp_key = :xp_key
                """

        # Выполнение запроса
        conn = current_app.ms.db(lname).connect()
        query = conn.execute(text(sql), {"xp_key": xp_key})
        res = query.fetchall()

        if not res:
            return False
        return [dict(row) for row in res]

    @staticmethod
    def insert_scholarship_list_option(lname: str, data: dict) -> Union[bool, int]:
        try:
            conn = current_app.ms.db(lname).connect()

            # Вставляем записи для каждого trmid с xp_key и education_level
            sql_trmid = """
            INSERT INTO scholarship_list_options (xp_key, trmid, date_from, date_to, education_level, prep_struc_category_id)
            VALUES (:xp_key, :trmid, :date_from, :date_to, :education_level, :prep_struc_category_id)
            RETURNING id
            """

            # Начинаем транзакцию
            with conn.begin():
                inserted_ids = []
                # Вставляем данные с trmid
                for trmid in data["trmids"]:
                    result = conn.execute(
                        text(sql_trmid),
                        {
                            "xp_key": data["xp_key"],
                            "trmid": trmid,
                            "education_level": data["education_level"],
                            "date_to": data["date_to"],
                            "date_from": data["date_from"],
                            "prep_struc_category_id": data["prep_struc_category_id"],
                        },
                    )
                    new_id = result.fetchone()[0]
                    inserted_ids.append(new_id)
                return inserted_ids if inserted_ids else False
        except Exception as e:
            print("Error inserting scholarship list option: {e}".format(e=e))
            return False

    @staticmethod
    def delete_scholarship_list_option(
        lname: str, xp_key: int, education_level: int, prep_struc_category_id: int
    ) -> bool:
        sql = "DELETE FROM scholarship_list_options WHERE xp_key = :xp_key AND education_level = :education_level AND prep_struc_category_id = :prep_struc_category_id"
        conn = current_app.ms.db(lname).connect()
        query = conn.execute(
            text(sql),
            {
                "xp_key": xp_key,
                "education_level": education_level,
                "prep_struc_category_id": prep_struc_category_id,
            },
        )

        return query.rowcount > 0  # Возвращает True, если удалено больше 0 строк

    @staticmethod
    def get_scholarship_candidates(
        lname: str,
        xp_key: int,
        mid: int,
        gid: int,
        education_level: int,
        scholarship_type: str,
    ) -> Union[bool, List[Dict[str, Any]]]:
        # Получаем параметры запроса
        filters = []
        params = {}

        if xp_key:
            filters.append("xp_key = :xp_key")
            params["xp_key"] = xp_key
        if mid:
            filters.append("mid = :mid")
            params["mid"] = mid
        if gid:
            filters.append("gid = :gid")
            params["gid"] = gid
        if education_level:
            filters.append("education_level = :education_level")
            params["education_level"] = education_level
        if scholarship_type:
            filters.append("scholarship_type = :scholarship_type")
            params["scholarship_type"] = scholarship_type

        sql = "SELECT * FROM scholarship_candidates WHERE 1=1"
        if filters:
            sql += " AND " + " AND ".join(filters)

        # Выполнение запроса
        conn = current_app.ms.db(lname).connect()
        query = conn.execute(text(sql), params)
        res = query.fetchall()

        if not res:
            return False
        return [dict(row) for row in res]

    @staticmethod
    def create_scholarship_candidate(lname: str, data: dict):
        sql = """
            INSERT INTO scholarship_candidates (xp_key, mid, gid, education_level, scholarship_type)
            VALUES (:xp_key, :mid, :gid, :education_level, :scholarship_type)
            ON CONFLICT (xp_key, mid, gid, education_level) 
            DO UPDATE SET scholarship_type = EXCLUDED.scholarship_type
            RETURNING scholarship_candidates_id, xp_key, mid, gid, education_level, scholarship_type
            """

        conn = current_app.ms.db(lname).connect()
        query = conn.execute(text(sql), data)
        result = query.fetchone()
        if not result:
            return False
        return result

    @staticmethod
    def delete_scholarship_candidate(lname: str, scholarship_candidates_id: int):
        sql = "DELETE FROM scholarship_candidates WHERE scholarship_candidates_id = :scholarship_candidates_id"

        # Выполнение запроса
        conn = current_app.ms.db(lname).connect()
        result = conn.execute(
            text(sql), {"scholarship_candidates_id": scholarship_candidates_id}
        )
        if result.rowcount == 0:
            return False
        return True

    @staticmethod
    def get_scholarship_candidates_list(
        lname: str,
        xp_key: int,
        education_level: int,
        scholarship_type: str,
        prep_struc_category_id: int,
        prep_levels: Tuple[int] = (),
    ) -> Union[bool, List[Dict[str, Any]]]:
        where = ""
        if scholarship_type:
            where += " AND sc.scholarship_type = :scholarship_type "

        if prep_levels:
            # Преобразование кортежа в строку для IN фильтра
            placeholders = ", ".join(
                [":prep_level_{i}".format(i=i) for i in range(len(prep_levels))]
            )
            where += " AND dc.preparation_level IN ({placeholders}) ".format(
                placeholders=placeholders
            )

        # Базовый SQL запрос
        sql = """
            SELECT 
                sc.*, 
				p.lastname || ' ' || p.firstname || ' ' || p.patronymic AS student_name, 
                ey.number yeducationyearid,
                ey.name yeducationyear, 
                sc.scholarship_type
            FROM degree_candidates3(:xp_key, :education_level, :prep_struc_category_id) dc
            JOIN scholarship_candidates sc 
                ON dc.mid = sc.mid AND sc.xp_key = :xp_key AND sc.education_level = :education_level
			JOIN people p ON p.mid = sc.mid
            LEFT JOIN groupname g 
                ON sc.gid = g.gid AND g.cid = -1
            LEFT JOIN educationyears ey 
                ON ey.number = g.year
            LEFT JOIN groupuser_transfer gt
                ON sc.mid = gt.mid
                AND gt.when = (
                    SELECT MAX(gut.when)
                    FROM groupuser_transfer gut
                    WHERE gut.mid = sc.mid
                )
            left join xp_status xs on xs.xp_key = gt.xp_status
			JOIN xp_personal_file xpf ON xpf.mid = sc.mid
            JOIN prep_struc_category psc ON psc.cid = xpf.category
            LEFT OUTER JOIN military_rank mr ON mr.id_mil_rank = xpf."MilitaryRank"
            WHERE dc.average_grade > 0 
              AND dc.three_count = 0 
              AND dc.two_count = 0 
              AND dc.education_level = :education_level
              AND dc.prep_struc_category_id = :prep_struc_category_id
              AND xs."Type" != 'Отчисление'
              AND xs."Type" != 'Выпуск'
              AND dc.five_percentage > 50.0
			  AND (sc.scholarship_type IS NULL OR sc.scholarship_type <> 'Нет')
			  {where}
            """.format(
            where=where
        )

        # Выполнение запроса
        conn = current_app.ms.db(lname).connect()
        stmt = text(sql)
        stmt = stmt.bindparams(
            xp_key=xp_key,
            education_level=education_level,
            prep_struc_category_id=prep_struc_category_id,
        )

        for i, level in enumerate(prep_levels):
            stmt = stmt.bindparams(**{"prep_level_{i}".format(i=i): level})

        if scholarship_type:
            stmt = stmt.bindparams(scholarship_type=scholarship_type)

        query = conn.execute(stmt)
        result = [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

        # Преобразование Decimal в float
        for row in result:
            for key, value in row.items():
                if isinstance(value, Decimal):
                    row[key] = float(value)
        return result

    @staticmethod
    def get_candidates_info(
        lname: str,
        mid: int,
        gid: int,
        xp_key: int,
        education_level: int,
        prep_struc_category_id: int,
    ):
        # Базовый SQL запрос
        sql = """
            SELECT 
                tg.mid, 
                p.lastname AS lastname, 
                p.firstname AS firstname, 
                p.patronymic AS patronymic, 
                gn.gid, 
                gn.name::TEXT AS groupname,
                COUNT(CASE WHEN NULLIF(tg.grade, '')::NUMERIC = 5 THEN 1 END)::INTEGER AS five_count,
                100.0 * COUNT(CASE WHEN NULLIF(tg.grade, '')::NUMERIC = 5 THEN 1 END) / NULLIF(COUNT(tg.grade), 0) AS five_percentage,
                COUNT(CASE WHEN NULLIF(tg.grade, '')::NUMERIC = 4 THEN 1 END)::INTEGER AS four_count,
                100.0 * COUNT(CASE WHEN NULLIF(tg.grade, '')::NUMERIC = 4 THEN 1 END) / NULLIF(COUNT(tg.grade), 0) AS four_percentage,
                ROUND(SUM(CAST(tg.grade AS INTEGER))::NUMERIC / NULLIF(COUNT(CAST(tg.grade AS INTEGER)), 0), 2) AS average_grade,
                ps.education_level,
                ps.preparation_level,
                ps.name::TEXT AS name,
                psc.cid AS prep_struc_category_id,
                psc.name AS prep_struc_category,
                gh.year AS kurs,
                mr.title AS mil_rank,
                sc.scholarship_type AS scholarship_type
            FROM term_grades tg
            JOIN courses c ON c.cid = tg.cid
            JOIN groupname gn ON gn.gid = :gid
            JOIN group_history gh ON gh.gid = gn.gid AND gh.school_year = :xp_key 
            JOIN militaryprofession mp ON gn.f_militaryprofession = mp.mpid
            JOIN edu_direction ed ON mp.edu_direct_id = ed.edu_direct_id
            JOIN preparation_structure ps ON ed.program_level = ps.ps_id
            JOIN people p ON p.mid = tg.mid
            LEFT JOIN xp_personal_file xpf ON xpf.mid = p.mid
            LEFT JOIN military_rank mr ON xpf."MilitaryRank" = mr.id_mil_rank
            LEFT JOIN prep_struc_category psc ON psc.cid = xpf.category
            LEFT JOIN scholarship_candidates sc ON sc.xp_key = :xp_key AND sc.mid = tg.mid AND sc.gid = gn.gid AND sc.education_level = :education_level 
            WHERE tg.mid = :mid
                AND tg.trmid IN (
                SELECT DISTINCT t.trmid
                FROM scholarship_list_options t
                WHERE t.xp_key = :xp_key  
                AND t.trmid IS NOT NULL
                AND t.education_level = :education_level
                AND t.prep_struc_category_id = :prep_struc_category_id
            )
            AND tg.jcid IN (
                SELECT DISTINCT jc.jcid
                FROM journalcertification jc
                JOIN nnz_schedule s ON s.sheid = jc.f_nnz_schedule
                JOIN eventtools e ON e.typeid = s.pair_type_id 
                    AND CASE WHEN 3 = 3 THEN e.processtypeid IN (3, 62) WHEN 3 = 8 THEN e.processtypeid = 8 END
                JOIN nnz_weeks w ON w.week_id = jc.f_nnz_weeks
                JOIN term_weeks tw ON tw.week_id = w.week_id
                JOIN terms t ON tw.trmid = t.trmid
                JOIN groupname g ON g.gid = s.gid
                JOIN group_history gh ON gh.gid = g.gid
                LEFT JOIN groupname g2 ON g2.gid = g.owner_gid
                LEFT JOIN group_history gh2 ON gh2.gid = g2.gid
                WHERE t.trmid IN (
                    SELECT DISTINCT slo.trmid
                    FROM scholarship_list_options slo
                    WHERE slo.xp_key = :xp_key 
                    AND slo.trmid IS NOT NULL
                    AND slo.education_level = :education_level
                    AND slo.prep_struc_category_id = :prep_struc_category_id
                )
                and coalesce(gh.school_year, gh2.school_year) = :xp_key
            )
            AND tg.grade IS NOT NULL  -- Убираем NULL значения
            AND tg.grade ~ '^[0-9]+(\.[0-9]+)?$'  -- Фильтруем только числовые значения
            GROUP BY tg.mid, gn.gid, gn.name, p.lastname, p.firstname, p.patronymic, ps.education_level, ps.preparation_level, 
                ps.name, psc.cid, psc.name, gh.year, mr.title, sc.scholarship_type
                """
        # Выполнение запроса
        conn = current_app.ms.db(lname).connect()
        stmt = text(sql)
        stmt = stmt.bindparams(
            mid=mid,
            gid=gid,
            xp_key=xp_key,
            education_level=education_level,
            prep_struc_category_id=prep_struc_category_id,
        )

        query = conn.execute(stmt)
        res = query.fetchone()
        if not res:
            return False
        else:
            return {
                "mid": res["mid"],
                "lastname": res["lastname"],
                "firstname": res["firstname"],
                "patronymic": res["patronymic"],
                "gid": res["gid"],
                "groupname": res["groupname"],
                "five_count": int(res["five_count"]),
                "five_percentage": float(res["five_percentage"]),
                "four_count": int(res["four_count"]),
                "four_percentage": float(res["four_percentage"]),
                "average_grade": float(res["average_grade"]),
                "education_level": int(res["education_level"]),
                "preparation_level": int(res["preparation_level"]),
                "name": res["name"],
                "prep_struc_category_id": int(res["prep_struc_category_id"]),
                "prep_struc_category": res["prep_struc_category"],
                "kurs": int(res["kurs"]),
                "mil_rank": res["mil_rank"],
                "scholarship_type": res["scholarship_type"],
            }

    @staticmethod
    def get_candidates_honours(
        lname: str,
        mid: int,
        xp_key: int,
        education_level: int,
        prep_struc_category_id: int,
    ):
        # Базовый SQL запрос
        sql = """
            WITH RECURSIVE indicator_tree AS (
                -- Шаг 1: Получаем записи первого уровня, где id_group = 0
                SELECT 
                    pi.id AS current_id,
                    pi.title AS current_title,
                    pi.id_group AS previous_id_group,
                    pi.id AS first_level_id,
                    pi.title AS first_level_title,
                    1 AS level
                FROM personnel_indicators pi
                JOIN rating_schema rs ON rs.id = pi.schema
                WHERE rs.stipend IS TRUE  
                  AND pi.id_group = 0
            
                UNION ALL
            
                -- Шаг 2: Рекурсивно получаем записи следующих уровней, где id_group = current_id предыдущего уровня
                SELECT 
                    pi.id AS current_id,
                    pi.title AS current_title,
                    pi.id_group AS previous_id_group,
                    it.first_level_id,
                    it.first_level_title,
                    it.level + 1 AS level
                FROM personnel_indicators pi
                JOIN rating_schema rs ON rs.id = pi.schema
                JOIN indicator_tree it ON pi.id_group = it.current_id
                WHERE rs.stipend IS TRUE
            ),
            final_table AS (
                SELECT 
                  it.first_level_id,
                  it.first_level_title,
                  it.current_id AS indicator_id,
                  pi2.name_for_report AS indicator_title,
                  it.previous_id_group AS previous_level_id,
                  pi.name_for_report AS previous_level_title,
                  it.level
                FROM indicator_tree it
                LEFT JOIN personnel_indicators pi ON it.previous_id_group = pi.id
                LEFT JOIN personnel_indicators pi2 ON it.current_id = pi2.id
                WHERE pi2.forgrant IS TRUE
                ORDER BY it.first_level_id, it.level
            ),
            balls AS (
                SELECT 
                    ft.previous_level_id, 
                    ft.previous_level_title, 
                    ft.indicator_id, 
                    ft.indicator_title, 
                    COALESCE(prv.value, 0) AS value -- Если значения нет, подставляем 0
                FROM final_table ft
                LEFT JOIN personnel_rating_values prv 
                    ON prv.indicator = ft.indicator_id
                    AND prv.mid = :mid
                    AND prv.value IS NOT NULL 
                    AND prv.value != 0
                    AND prv.periodend BETWEEN 
                        (SELECT DISTINCT slo.date_from 
                         FROM scholarship_list_options slo 
                         WHERE slo.xp_key = :xp_key
                           AND slo.date_from IS NOT NULL 
                           AND slo.education_level = :education_level
                           AND slo.prep_struc_category_id = :prep_struc_category_id) 
                        AND 
                        (SELECT DISTINCT slo.date_to 
                         FROM scholarship_list_options slo 
                         WHERE slo.xp_key = :xp_key
                           AND slo.date_from IS NOT NULL 
                           AND slo.education_level = :education_level
                           AND slo.prep_struc_category_id = :prep_struc_category_id)
                WHERE ft.level = (SELECT MAX(ft.level) FROM final_table ft)
                GROUP BY ft.previous_level_id, ft.previous_level_title, ft.indicator_id, ft.indicator_title, prv.value
            ), 
            summ_balls AS (
                select b.previous_level_id, sum(b.value) as sum_value
                from balls b
                GROUP BY b.previous_level_id
            )
            select b.*, sb.sum_value
            from balls b
            join summ_balls sb ON sb.previous_level_id = b.previous_level_id
                    """
        # Выполнение запроса
        conn = current_app.ms.db(lname).connect()
        stmt = text(sql)
        stmt = stmt.bindparams(
            mid=mid,
            xp_key=xp_key,
            education_level=education_level,
            prep_struc_category_id=prep_struc_category_id,
        )

        query = conn.execute(stmt)
        result = [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

        # Преобразование Decimal в float
        for row in result:
            for key, value in row.items():
                if isinstance(value, Decimal):
                    row[key] = float(value)
        return result

    @staticmethod
    def get_columns_honours(
        lname: str,
    ):
        # Базовый SQL запрос
        sql = """
            WITH RECURSIVE indicator_tree AS (
              -- Шаг 1: Получаем записи первого уровня, где id_group = 0
              SELECT 
               pi.id AS current_id,
               pi.title AS current_title,
               pi.id_group AS previous_id_group,
               pi.id AS first_level_id,
               pi.title AS first_level_title,
               1 AS level
              FROM personnel_indicators pi
              JOIN rating_schema rs ON rs.id = pi.schema
              WHERE rs.stipend IS TRUE  
                AND pi.id_group = 0
              UNION ALL
              -- Шаг 2: Рекурсивно получаем записи следующих уровней, где id_group = current_id предыдущего уровня
              SELECT 
               pi.id AS current_id,
               pi.title AS current_title,
               pi.id_group AS previous_id_group,
               it.first_level_id,
               it.first_level_title,
               it.level + 1 AS level
              FROM personnel_indicators pi
              JOIN rating_schema rs ON rs.id = pi.schema
              JOIN indicator_tree it ON pi.id_group = it.current_id
              WHERE rs.stipend IS TRUE
             ),
             final_table AS (
              SELECT 
               it.first_level_id,
               it.first_level_title,
               it.current_id AS indicator_id,
               it.current_title AS indicator_title,
               it.previous_id_group AS previous_level_id,
               pi.name_for_report AS previous_level_title,
               it.level
              FROM indicator_tree it
              LEFT JOIN personnel_indicators pi ON it.previous_id_group = pi.id
              LEFT JOIN personnel_indicators pi2 ON it.current_id = pi2.id
              WHERE pi2.forgrant IS TRUE
              ORDER BY it.first_level_id, it.level
             )
             SELECT DISTINCT ft.previous_level_id, ft.previous_level_title FROM final_table ft
            """
        # Выполнение запроса
        conn = current_app.ms.db(lname).connect()
        stmt = text(sql)
        query = conn.execute(stmt)
        result = [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

        # Преобразование Decimal в float
        for row in result:
            for key, value in row.items():
                if isinstance(value, Decimal):
                    row[key] = float(value)
        return result

    @staticmethod
    def check_levels_of_education(lname: str, xp_key: int):
        # Базовый SQL запрос
        sql = """
            -- Шаг 1: Создаем временную таблицу для хранения всех данных по группам
            CREATE TEMP TABLE all_group_data (
                mid INT,
                cid INT,
                gid INT
            ) ON COMMIT DROP;  -- Удалить таблицу при завершении транзакции
            -- Шаг 2: Создаем временную таблицу с валидными jcid
            DROP TABLE IF EXISTS valid_jsid;
            CREATE TEMP TABLE valid_jsid AS
            SELECT DISTINCT coalesce(g2.gid, g.gid) AS gid, coalesce(g.name, g2.name) AS groupname
            FROM journalcertification jc
            JOIN nnz_schedule s ON s.sheid = jc.f_nnz_schedule
            JOIN eventtools e ON e.typeid = s.pair_type_id 
                AND CASE WHEN 3 = 3 THEN e.processtypeid IN (3, 62) WHEN 3 = 8 THEN e.processtypeid = 8 END
            JOIN nnz_weeks w ON w.week_id = jc.f_nnz_weeks
            JOIN term_weeks tw ON tw.week_id = w.week_id
            JOIN terms t ON tw.trmid = t.trmid
            JOIN groupname g ON g.gid = s.gid
            JOIN group_history gh ON gh.gid = g.gid
            LEFT JOIN groupname g2 ON g2.gid = g.owner_gid
            LEFT JOIN group_history gh2 ON gh2.gid = g2.gid
            WHERE t.term_begin::date BETWEEN (
                            SELECT (sy.enddate - INTERVAL '2 year')::date 
                            FROM school_year sy
                            WHERE sy.xp_key = :xp_key
                        ) AND (
                            SELECT sy.enddate
                            FROM school_year sy
                            WHERE sy.xp_key = :xp_key
                        )
                        and coalesce(gh.school_year, gh2.school_year) = :xp_key;			
            SELECT DISTINCT
                psc.cid AS prep_struc_category_id,
                psc.name AS prep_struc_category
            FROM valid_jsid vj
            JOIN groupname g ON g.gid = vj.gid
            JOIN groupuser gu ON gu.gid = g.gid
            JOIN militaryprofession mp ON g.f_militaryprofession = mp.mpid
            JOIN edu_direction ed ON mp.edu_direct_id = ed.edu_direct_id
            JOIN preparation_structure ps ON ed.program_level = ps.ps_id
            JOIN xp_personal_file e ON e.mid = gu.mid 
            JOIN prep_struc_category psc ON psc.cid = e.category
            WHERE ps.education_level IN (1, 2)
            
            UNION ALL
            
            -- Добавляем запись с СПО, если есть education_level = 2
            SELECT DISTINCT
                0 AS prep_struc_category_id,
                'СПО' AS prep_struc_category
            FROM valid_jsid vj
            JOIN groupname g ON g.gid = vj.gid
            JOIN groupuser gu ON gu.gid = g.gid
            JOIN militaryprofession mp ON g.f_militaryprofession = mp.mpid
            JOIN edu_direction ed ON mp.edu_direct_id = ed.edu_direct_id
            JOIN preparation_structure ps ON ed.program_level = ps.ps_id
            WHERE ps.education_level = 2;
                """
        # Выполнение запроса
        conn = current_app.ms.db(lname).connect()
        stmt = text(sql)
        stmt = stmt.bindparams(xp_key=xp_key)
        query = conn.execute(stmt)
        result = [dict(zip(tuple(query.keys()), i)) for i in query.cursor]

        # Преобразование Decimal в float
        for row in result:
            for key, value in row.items():
                if isinstance(value, Decimal):
                    row[key] = float(value)
        return result


class DegreeCandidatesTask:
    def __init__(self, cname):
        self.lname = cname

    def create_degree_candidates_task(self, task_id, mid, status):
        try:
            conn = current_app.ms.db(self.lname).connect()
            sql = """
                insert into degree_candidates_task (task_id, mid, created_at, status) 
                values (:task_id, :mid, NOW(),:status)
                """
            stmt = text(sql)
            stmt = stmt.bindparams(task_id=task_id)
            stmt = stmt.bindparams(mid=mid)
            stmt = stmt.bindparams(status=status)
            query = conn.execute(stmt)
            return True
        except Exception as e:
            print(e)
            return False

    def update_status_degree_candidates_task(self, task_id, status):
        conn = current_app.ms.db(self.lname).connect()
        sql = """
            UPDATE degree_candidates_task
            SET status = :status
            WHERE task_id = :task_id
            """

        stmt = text(sql)
        stmt = stmt.bindparams(task_id=task_id, status=status)
        conn.execute(stmt)

    def update_data_degree_candidates_task(self, task_id, status, data):
        conn = current_app.ms.db(self.lname).connect()
        sql = """
            UPDATE degree_candidates_task
            SET status = :status, data = :data
            WHERE task_id = :task_id
            """

        stmt = text(sql)
        stmt = stmt.bindparams(
            task_id=task_id,
            status=status,
            data=json.dumps(data, ensure_ascii=False),
        )
        conn.execute(stmt)

    def get_degree_candidates_task(self, task_id):
        conn = current_app.ms.db(self.lname).connect()
        sql = """
            SELECT * FROM degree_candidates_task
            WHERE task_id = :task_id
            """
        stmt = text(sql)
        stmt = stmt.bindparams(task_id=task_id)
        query = conn.execute(stmt)
        result = query.first()
        return result
