from flask import Flask, jsonify, request, current_app, g
from flask_restful import Resource, Api
from sqlalchemy import create_engine
from sqlalchemy.sql import text
from flask import Blueprint
from LMSAPI.api.Views.TokenAPI import auth
from LMSAPI.api.Models.User import User
from LMSAPI.api.Models.Cathedra import Cathedra
from LMSAPI.api.Models.Group import Group
from flask_cors import CORS

from LMSAPI.api.utils.access_utils import user_permission_modes

# https://stackoverflow.com/questions/39769666/flask-how-to-use-app-context-inside-blueprints
# Use app context from blueprint

group_api = Blueprint("group_api", __name__)

CORS(group_api)


@group_api.route(
    "/lms/api/v1.0/<lname>/group/<int:cid>/<int:fid>/<int:yid>/<int:educationyid>",
    methods=["GET"],
)
def getGroups(lname, cid, fid, yid, educationyid):
    """GET with ID of Cathedra to get list of groups"""
    result = Group(lname).getGroupList(lname, cid, fid, yid, educationyid)

    if result is None:
        return jsonify(success=False), 404

    return jsonify([fa.serialize() for fa in result])


@group_api.route(
    "/lms/api/v1.0/<lname>/group/teacher/<int:cid>/<int:trimid>", methods=["GET"]
)
@auth.login_required
def getGroupsForTeacher(lname, cid, trimid):
    result = Group(lname).getGroupListForTeacher(lname, cid, trimid)

    if result is None:
        return jsonify(success=False), 404

    return jsonify([fa.serialize() for fa in result])


@group_api.route(
    "/lms/api/v1.0/<string:lname>/group/<int:school_year_id>/<int:trim_id>",
    methods=["GET"],
)
@auth.login_required
# @user_permission_modes("Режимы", "Журнал учета учебных занятий", ["Нет"])
def get_groups_journal(lname: str, school_year_id: int, trim_id: int):
    filters = request.args
    faculty_id = filters.get("faculty_id")

    if faculty_id and not faculty_id.isdigit():
        return jsonify(msg="faculty_id must be digit"), 400

    cathedra_id = filters.get("cathedra_id")

    if cathedra_id and not cathedra_id.isdigit():
        return jsonify(msg="cathedra_id must be digit"), 400

    result = Group(lname).getGroupListJournal(
        lname,
        cathedra_id=cathedra_id,
        trim_id=trim_id,
        school_year_id=school_year_id,
        faculty_id=faculty_id,
    )

    return jsonify(result)


@group_api.route(
    "/lms/api/v1.0/<lname>/group/<int:faculty_id>/<int:course>/<chosen_date>",
    methods=["GET"],
)
@auth.login_required
def getGroupsByCourse(lname, faculty_id, course, chosen_date):
    result = Group.getGroupListByCourse(lname, faculty_id, course, chosen_date)

    if result is None:
        return jsonify(success=False), 404

    return jsonify(result)


@group_api.route("/lms/api/v1.0/<lname>/group", methods=["GET"])
@auth.login_required
def getAllGroups(lname):
    result = Group(lname).getAllGroups(lname)

    if result is None:
        return jsonify(success=False), 404

    return jsonify([fa.serialize() for fa in result])


@group_api.route("/lms/api/v1.0/<lname>/group/<int:id>", methods=["GET"])
@auth.login_required
def getGroup(lname, id):
    """GET with ID to get details for a Group"""

    result = Group(lname).getGroup(lname, id)

    if result is None:
        return jsonify(success=False), 404

    return jsonify(result)


@group_api.route("/lms/api/v1.0/<lname>/group/<int:id>/students", methods=["GET"])
@auth.login_required
def getGroupStudents(lname, id):
    """GET with ID to get details for a Group"""

    result = Group(lname).getGroupStudents(lname, id)

    if result is None:
        return jsonify(success=False), 404

    return jsonify(result)


@group_api.route("/lms/api/v1.0/<lname>/role/group", methods=["GET"])
@auth.login_required
@user_permission_modes(
    "Режимы",
    "Ввод индивидуальных достижений." and "Рейтинг распределение и цифровой портрет.",
    ["Нет"],
)
def get_groups_by_role(lname):
    xp_key = request.args.get("xp_key", default=0, type=int)
    idfaculty = request.args.get("idfaculty", default=0, type=int)
    idcathedra = request.args.get("idcathedra", default=0, type=int)
    result = Group(lname).get_groups_by_role(lname, xp_key, idfaculty, idcathedra)

    if result is None:
        return (
            jsonify(
                success=False,
            ),
            404,
        )
    if not result:
        return (
            jsonify(
                success=False,
                message="""Не найдены группы для пользователя с правами "Ввод индивидуальных достижений." или "Рейтинг распределение и цифровой портрет." и фильтрами по факультету, кафедре и году""",
            ),
            404,
        )

    return jsonify(result)


@group_api.route("/lms/api/v1.0/<lname>/admin/group", methods=["GET"])
@auth.login_required
@user_permission_modes(
    "Режимы",
    "Ввод индивидуальных достижений. Администратор."
    and "Рейтинг распределение и цифровой портрет. Администратор.",
    ["Нет"],
)
def get_groups_by_admin(lname):
    xp_key = request.args.get("xp_key", default=0, type=int)
    idfaculty = request.args.get("idfaculty", default=0, type=int)
    idcathedra = request.args.get("idcathedra", default=0, type=int)
    result = Group(lname).get_groups_by_admin(lname, xp_key, idfaculty, idcathedra)

    if result is None:
        return (
            jsonify(
                success=False,
            ),
            404,
        )
    if not result:
        return (
            jsonify(
                success=False,
                message="""Не найдены группы с фильтрами по факультету, кафедре и году""",
            ),
            404,
        )

    return jsonify(result)
