# coding=UTF-8
import sys
from decimal import Decimal

from flask import jsonify, request, json, g
from flask import Blueprint

from LMSAPI.api.Models.EnrollAnswers import EnrollAnswers
from LMSAPI.api.Models.EnrollQuestions import EnrollQuestions
from LMSAPI.api.Models.QuestionAnswers import QuestionAnswers
from LMSAPI.api.Models.QuestionSet import QuestionSet, QuestionSetContent
from LMSAPI.api.Views.TokenAPI import auth
from flask_cors import CORS

questionset_api = Blueprint("questionset_api", __name__)

CORS(questionset_api)


@questionset_api.route(
    "/lms/api/v1.0/<lname>/questionset",
    methods=["GET"],
)
@auth.login_required
def get_questionset_all(lname):
    cid = request.args.get("cid", default=None, type=int)
    private = request.args.get("private", default=None)
    oid = request.args.get("oid", default=None, type=int)

    # Корректная обработка значения параметра private
    if private is not None:
        private = private.lower() in ["true", "1", "t", "y", "yes"]

    result = QuestionSet().get_questionset_all(lname, cid, private, oid)

    if result is None:
        return jsonify(success=False), 404

    return jsonify(result)


@questionset_api.route(
    "/lms/api/v1.0/<lname>/questionset/<int:id>/question/answers",
    methods=["GET"],
)
@auth.login_required
def get_question_and_answer_by_test(lname, id):
    result = QuestionSet().get_question_and_answer_by_test(lname, id, g.user.isTeacher)

    if result is None:
        return jsonify(success=False), 404

    # Convert Decimal to float
    for row in result:
        for key, value in row.items():
            if isinstance(value, Decimal):
                row[key] = float(value)

    return jsonify(result)


@questionset_api.route(
    "/lms/api/v1.0/<lname>/questionset/<int:id>",
    methods=["GET"],
)
@auth.login_required
def get_questionset_by_id(lname, id: int):
    result = QuestionSet().get_questionset_by_id(lname, id)

    if result is None:
        return jsonify(success=False), 404

    for key, value in result.items():
        if isinstance(value, Decimal):
            result[key] = float(value)

    availability_answers = QuestionAnswers().get_availability_answers_test_questions(
        lname, g.user.mid, id
    )
    if availability_answers:
        result["availability_answers"] = True
    else:
        result["availability_answers"] = False

    return jsonify(result)


@questionset_api.route(
    "/lms/api/v1.0/<lname>/questionset",
    methods=["POST"],
)
@auth.login_required
def create_questionset(lname):
    data = request.data
    if not data:
        return jsonify({"error": "No data provided"}), 400

    if sys.version_info[0] < 3:
        data = json.loads(data)
    else:
        data = json.loads(data.decode("utf-8"))

    # Проверка на наличие всех обязательных полей
    missing_fields = [
        field
        for field in ["name", "cid", "private", "description", "oid"]
        if field not in data
    ]
    if missing_fields:
        return (
            jsonify(
                {
                    "error": "Missing required fields: {message}".format(
                        message=", ".join(missing_fields)
                    )
                }
            ),
            400,
        )

    result = QuestionSet().create_questionset(lname, data)

    if result is None:
        return jsonify(success=False), 404

    return jsonify(success=True), 201


@questionset_api.route(
    "/lms/api/v1.0/<lname>/questionset/<int:id>",
    methods=["PUT"],
)
@auth.login_required
def update_questionset(lname, id: int):
    data = request.data
    if not data:
        return jsonify({"error": "No data provided"}), 400

    if sys.version_info[0] < 3:
        data = json.loads(data)
    else:
        data = json.loads(data.decode("utf-8"))

    result = QuestionSet().update_questionset(lname, id, data)

    if result is None:
        return jsonify(success=False), 404

    return jsonify(success=True), 201


@questionset_api.route(
    "/lms/api/v1.0/<lname>/questionset/<int:id>",
    methods=["DELETE"],
)
@auth.login_required
def delete_questionset(lname, id: int):
    result = QuestionSet().delete_questionset(lname, id)

    if result is None:
        return jsonify(success=False), 404
    return jsonify(success=True), 200


@questionset_api.route(
    "/lms/api/v1.0/<lname>/questionset/content",
    methods=["GET"],
)
@auth.login_required
def get_questionset_content_all(lname):
    id_set = request.args.get("id_set", default=None, type=int)
    id_question = request.args.get("id_question", default=None, type=int)

    result = QuestionSetContent().get_questionset_content_all(
        lname, id_question, id_set
    )
    if result is None:
        return jsonify(success=False), 404

    return jsonify(result)


@questionset_api.route(
    "/lms/api/v1.0/<lname>/questionset/content",
    methods=["POST"],
)
@auth.login_required
def create_questionset_content(lname):
    data = request.data
    if not data:
        return jsonify({"error": "No data provided"}), 400

    if sys.version_info[0] < 3:
        data = json.loads(data)
    else:
        data = json.loads(data.decode("utf-8"))

    # Проверка на наличие всех обязательных полей
    missing_fields = [
        field for field in ["id_question", "id_set", "sortnum"] if field not in data
    ]
    if missing_fields:
        return (
            jsonify(
                {
                    "error": "Missing required fields: {message}".format(
                        message=", ".join(missing_fields)
                    )
                }
            ),
            400,
        )

    result = QuestionSetContent().create_questionset_content(lname, data)

    if result is None:
        return jsonify(success=False), 404

    return jsonify(success=True), 201


@questionset_api.route(
    "/lms/api/v1.0/<lname>/questionset/content/<int:id_question>/<int:id_set>",
    methods=["DELETE"],
)
@auth.login_required
def delete_questionset_content(lname, id_question: int, id_set: int):
    result = QuestionSetContent().delete_questionset_content(lname, id_question, id_set)

    if result is None:
        return jsonify(success=False), 404
    return jsonify(success=True), 200


@questionset_api.route(
    "/lms/api/v1.0/<lname>/questionset/<int:id>/user/statistic",
    methods=["GET"],
)
@auth.login_required
def get_statistics_by_questionset(lname, id):
    all_question_answers = QuestionSet().get_question_and_answer_by_test(
        lname, id, True
    )
    count_question = len(all_question_answers)
    text_question = 0
    correct_question_answers = 0
    incorrect_question_answers = 0

    for question in all_question_answers:
        user_question_answers = QuestionAnswers().get_question_answers_all(
            lname, hwid=None, mid=g.user.mid, questionset=id, question=question["id"]
        )
        if question["question_type"] == 1:
            # Проверяем, что есть ровно один пользовательский ответ
            if len(user_question_answers) == 1:
                user_answer_id = user_question_answers[0]["answer_id"]

                # Находим ответ с таким же answer_id в вопросе и проверяем его корректность
                correct_answer_found = False
                for answer in question["answers"]:
                    if answer["id"] == user_answer_id and answer.get("correct") is True:
                        correct_answer_found = True
                        break

                # Увеличиваем счетчик правильных ответов, если найден правильный ответ
                if correct_answer_found:
                    correct_question_answers += 1
                else:
                    incorrect_question_answers += 1
            else:
                return (
                    jsonify(
                        success=False,
                        message="У данного типа вопроса не может быть несколько ответов",
                    ),
                    404,
                )

        elif question["question_type"] == 2:
            # Для типа вопроса 1, пользователю разрешено выбрать несколько правильных ответов

            # Находим все правильные ответы для этого вопроса
            correct_answers = {
                answer["id"]
                for answer in question["answers"]
                if answer.get("correct") is True
            }
            user_answers = {
                user_answer["answer_id"] for user_answer in user_question_answers
            }

            # Проверка: все ли пользовательские ответы являются правильными и их количество соответствует
            if user_answers == correct_answers:
                correct_question_answers += 1
            else:
                incorrect_question_answers += 1

        elif question["question_type"] == 3:
            # Текстовый ответ, не возможно узнать правильно ли ответил пользователь
            text_question += 1

        elif question["question_type"] == 4:
            # Вопрос типа 3: проверка порядка ответов

            is_correct = True  # Флаг для отслеживания правильности ответа на вопрос

            for user_answer in user_question_answers:
                # Получаем id ответа и место, которое пользователь выбрал для этого ответа
                user_answer_id = user_answer["answer_id"]
                user_answer_group = user_answer["answer_num"]

                # Находим правильный ответ с тем же id
                correct_answer = next(
                    (
                        answer
                        for answer in question["answers"]
                        if answer["id"] == user_answer_id
                    ),
                    None,
                )

                # Проверяем соответствие
                if (
                    not correct_answer
                    or correct_answer["answer_group"] != user_answer_group
                ):
                    is_correct = False
                    break

            if is_correct:
                correct_question_answers += 1
            else:
                incorrect_question_answers += 1

        elif question["question_type"] == 5:
            if len(question["answers"]) == 1:
                text_question += 1
            else:
                user_answer_id = user_question_answers[0]["answer_id"]

                # Находим ответ с таким же answer_id в вопросе и проверяем его корректность
                correct_answer_found = False
                for answer in question["answers"]:
                    if answer["id"] == user_answer_id and answer.get("correct") is True:
                        correct_answer_found = True
                        break

                # Увеличиваем счетчик правильных ответов, если найден правильный ответ
                if correct_answer_found:
                    correct_question_answers += 1
                else:
                    incorrect_question_answers += 1
        else:
            return (
                jsonify(
                    success=False,
                    message="Нет такого типа вопроса",
                ),
                404,
            )

    if count_question - text_question > 0:
        user_correct_percentage = (
            correct_question_answers / (count_question - text_question)
        ) * 100
    else:
        user_correct_percentage = 0

    return jsonify(
        {
            "count_all_question": count_question,
            "text_question": text_question,
            "multiple_choice_questions": count_question - text_question,
            "correct_question_answers": correct_question_answers,
            "incorrect_question_answers": incorrect_question_answers,
            "user_correct_percentage": user_correct_percentage,
        }
    )
