import threading
import uuid

from flask import Flask, jsonify, request, current_app, g
from flask import Blueprint

from LMSAPI.api.Models.Company import Company
from LMSAPI.api.Models.Group import Group
from flask_cors import CORS

from LMSAPI.api.Models.Rating import Rating
from LMSAPI.api.Models.ScholarshipListOptions import (
    ScholarshipListOptions,
    DegreeCandidatesTask,
)
from LMSAPI.api.Views.TokenAPI import auth

scholarship = Blueprint("scholarship", __name__, url_prefix="/lms/api/v1.0/<lname>")

CORS(scholarship)


def degree_candidates_task(app, lname, task_id, xp_key):
    with app.app_context():
        try:
            result = {}

            options = ScholarshipListOptions(
                lname
            ).get_education_level_and_prep_struc_category_id_option(
                lname=lname,
                xp_key=xp_key,
            )

            for option in options:
                if (
                    option["education_level"] == 1
                    and option["prep_struc_category_id"] == 1
                ):
                    result[
                        "scholarship_higher_education_cadet"
                    ] = ScholarshipListOptions(lname).get_scholarship_list(
                        lname, xp_key, 1, 1, (1, 2, 4, 5)
                    )
                elif (
                    option["education_level"] == 1
                    and option["prep_struc_category_id"] == 2
                ):
                    result[
                        "scholarship_higher_education_listener"
                    ] = ScholarshipListOptions(lname).get_scholarship_list(
                        lname, xp_key, 1, 2, (1, 2, 4, 5)
                    )
                elif (
                    option["education_level"] == 1
                    and option["prep_struc_category_id"] == 3
                ):
                    result[
                        "scholarship_higher_education_adjunct"
                    ] = ScholarshipListOptions(lname).get_scholarship_list(
                        lname, xp_key, 1, 3, (1, 2, 4, 5)
                    )
                elif option["education_level"] == 2:
                    result[
                        "scholarship_higher_education_adjunct"
                    ] = ScholarshipListOptions(lname).get_scholarship_list(
                        lname, xp_key, 2, 1, (3,)
                    )
                else:
                    pass
            result["columns_honours"] = ScholarshipListOptions(
                lname
            ).get_columns_honours(lname)
            DegreeCandidatesTask(lname).update_data_degree_candidates_task(
                task_id, "completed", result
            )

        except Exception as e:
            print(e)
            DegreeCandidatesTask(lname).update_status_degree_candidates_task(
                task_id, "failed"
            )


@scholarship.route(
    "/degree_candidates/<int:xp_key>",
    methods=["GET"],
)
@auth.login_required
def get_degree_candidates(lname, xp_key):
    if not Rating(lname).has_true_stipend():
        return (
            jsonify({"error": "Нет схем с полем stipend для подсчета достижений"}),
            404,
        )

    task_id = str(uuid.uuid4())  # Генерируем уникальный идентификатор задачи
    DegreeCandidatesTask(lname).create_degree_candidates_task(
        task_id, g.user.mid, "in_progress"
    )
    # Запускаем задачу в отдельном потоке
    app = current_app._get_current_object()
    thread = threading.Thread(
        target=degree_candidates_task,
        args=(app, lname, task_id, xp_key),
    )
    thread.start()
    return jsonify({"task_id": task_id}), 202


@scholarship.route("/degree_candidates_task/<string:task_id>", methods=["GET"])
@auth.login_required
def get_task_details(lname, task_id):
    task_id = uuid.UUID(task_id).hex
    degree_candidates_task = DegreeCandidatesTask(lname).get_degree_candidates_task(
        task_id
    )

    if not degree_candidates_task:
        return jsonify({"error": "Task not found"}), 404

    if degree_candidates_task and degree_candidates_task.mid != g.user.mid:
        return (
            jsonify({"error": "У данного пользователя нет доступа к этой информации"}),
            404,
        )

    if degree_candidates_task.status == "completed":
        return (
            jsonify(
                {
                    "status": degree_candidates_task.status,
                    "data": degree_candidates_task.data,
                }
            ),
            200,
        )
    elif degree_candidates_task.status == "failed":
        return jsonify({"status": degree_candidates_task.status}), 500

    return jsonify({"status": degree_candidates_task.status}), 200


@scholarship.route(
    "/degree_candidates/info/<int:mid>/<int:gid>/<int:xp_key>/<int:education_level>/<int:prep_struc_category_id>",
    methods=["GET"],
)
@auth.login_required
def get_candidates_info(
    lname, mid, gid, xp_key, education_level, prep_struc_category_id
):
    result = ScholarshipListOptions(lname).get_candidates_info(
        lname, mid, gid, xp_key, education_level, prep_struc_category_id
    )
    if result:
        result["ovu_name"] = Company().getSetting(lname, "OVU_Name")
        result["gou_short"] = Company().getSetting(lname, "GOU_Short")
        result["honours"] = ScholarshipListOptions(lname).get_candidates_honours(
            lname, mid, xp_key, education_level, prep_struc_category_id
        )
    return jsonify(result)


@scholarship.route("/scholarship_list_options", methods=["GET"])
@auth.login_required
def get_scholarship_list(lname):
    try:
        # Получаем параметры запроса
        id = request.args.get("id", type=int)
        trmid = request.args.get("trmid", type=int)
        xp_key = request.args.get("xp_key", type=int)
        education_level = request.args.get("education_level", type=int)

        final_result = {}
        if xp_key:
            education_levels = ScholarshipListOptions(lname).check_levels_of_education(
                lname=lname, xp_key=xp_key
            )
            final_result["education_levels"] = education_levels

        # Вызываем метод для получения данных
        result = ScholarshipListOptions(lname).get_scholarship_list_options(
            lname=lname,
            id=id,
            trmid=trmid,
            xp_key=xp_key,
            education_level=education_level,
        )

        # Если данные не найдены
        if not result:
            return jsonify({"error": "No records found"}), 404

        final_result["data"] = result

        # Возвращаем результат в формате JSON
        return jsonify(final_result), 200
    except Exception as e:
        print("Error in GET request: {e}".format(e=e))
        return jsonify({"error": str(e)}), 500


@scholarship.route(
    "/scholarship_list_options",
    methods=["POST"],
)
@auth.login_required
def insert_scholarship_list(lname):
    try:
        # Получаем данные из запроса
        data = request.json

        # Проверяем, что данные являются массивом
        if not isinstance(data, list):
            return jsonify({"error": "Invalid input format. Expected a list."}), 400

        inserted_ids = []

        # Обрабатываем каждый элемент массива
        for entry in data:
            if (
                "xp_key" in entry
                and "trmids" in entry
                and "education_level" in entry
                and "date_to" in entry
                and "date_from" in entry
                and "prep_struc_category_id" in entry
            ):
                ScholarshipListOptions(lname).delete_scholarship_list_option(
                    lname,
                    entry["xp_key"],
                    entry["education_level"],
                    entry["prep_struc_category_id"],
                )

                new_id = ScholarshipListOptions(lname).insert_scholarship_list_option(
                    lname, entry
                )
                if new_id:
                    inserted_ids.extend(new_id)
                else:
                    return jsonify({"error": "Failed to insert data."}), 500
            else:
                return jsonify({"error": "Invalid input data structure."}), 400

        return jsonify({"inserted_ids": inserted_ids}), 200
    except Exception as e:
        print("Error in POST request: {e}".format(e=e))
        return jsonify({"error": str(e)}), 500


@scholarship.route("/scholarship_candidates", methods=["GET"])
@auth.login_required
def get_scholarship_candidates(lname):
    try:
        xp_key = request.args.get("xp_key", type=int)
        mid = request.args.get("mid", type=int)
        gid = request.args.get("gid", type=int)
        education_level = request.args.get("education_level", type=int)
        scholarship_type = request.args.get("scholarship_type", type=str)

        result = ScholarshipListOptions(lname).get_scholarship_candidates(
            lname=lname,
            xp_key=xp_key,
            mid=mid,
            gid=gid,
            education_level=education_level,
            scholarship_type=scholarship_type,
        )

        if not result:
            return jsonify({"error": "No records found"}), 404
        return jsonify(result), 200
    except Exception as e:
        print("Error in GET request: {e}".format(e=e))
        return jsonify({"error": str(e)}), 500


@scholarship.route("/scholarship_candidates", methods=["POST"])
@auth.login_required
def create_scholarship_candidate(lname):
    data = request.json

    if not all(
        k in data
        for k in ("xp_key", "mid", "gid", "education_level", "scholarship_type")
    ):
        return jsonify({"message": "Missing required fields"}), 400

    # Проверка на допустимые значения scholarship_type
    allowed_scholarship_types = [
        "Президента РФ",
        "Правительства РФ",
        "МО РФ",
        "Исключить",
    ]
    if data["scholarship_type"] not in allowed_scholarship_types:
        return jsonify({"message": "Invalid scholarship_type value"}), 400

    scholarship_candidate = ScholarshipListOptions(lname).create_scholarship_candidate(
        lname, data
    )
    if not scholarship_candidate:
        return jsonify({"message": "Failed to create candidate"}), 500
    return (
        jsonify(dict(scholarship_candidate)),
        201,
    )


@scholarship.route(
    "/scholarship_candidates/<int:scholarship_candidates_id>", methods=["DELETE"]
)
def delete_scholarship_candidate(lname, scholarship_candidates_id):
    scholarship_candidate = ScholarshipListOptions(lname).delete_scholarship_candidate(
        lname, scholarship_candidates_id
    )
    if scholarship_candidate is False:
        return jsonify(success=False, message="Candidate not found"), 404
    return jsonify(success=True, message="Candidate deleted"), 200


@scholarship.route(
    "/scholarship_candidates/<type>/<int:xp_key>",
    methods=["GET"],
)
@auth.login_required
def get_scholarship_candidates_mo_list(lname, type, xp_key):
    if type == "mo":
        result = {
            "scholarship_candidates_higher_cadet": ScholarshipListOptions(
                lname
            ).get_scholarship_candidates_list(
                lname, xp_key, 1, "МО РФ", 1, (1, 2, 4, 5)
            ),
            "scholarship_candidates_higher_listener": ScholarshipListOptions(
                lname
            ).get_scholarship_candidates_list(
                lname, xp_key, 1, "МО РФ", 2, (1, 2, 4, 5)
            ),
            "scholarship_candidates_higher_adjunct": ScholarshipListOptions(
                lname
            ).get_scholarship_candidates_list(
                lname, xp_key, 1, "МО РФ", 3, (1, 2, 4, 5)
            ),
        }
    elif type == "government":
        president_scholarship_candidates_higher_cadet_listener = ScholarshipListOptions(
            lname
        ).get_scholarship_candidates_list(
            lname, xp_key, 1, "Президента РФ", 1, (1, 2, 4, 5)
        )
        for i in ScholarshipListOptions(lname).get_scholarship_candidates_list(
            lname, xp_key, 1, "Президента РФ", 2, (1, 2, 4, 5)
        ):
            president_scholarship_candidates_higher_cadet_listener.append(i)

        government_scholarship_candidates_higher_cadet_listener = (
            ScholarshipListOptions(lname).get_scholarship_candidates_list(
                lname, xp_key, 1, "Правительства РФ", 1, (1, 2, 4, 5)
            )
        )
        for i in ScholarshipListOptions(lname).get_scholarship_candidates_list(
            lname, xp_key, 1, "Правительства РФ", 2, (1, 2, 4, 5)
        ):
            government_scholarship_candidates_higher_cadet_listener.append(i)

        result = {
            "president_scholarship_candidates_higher_cadet_listener": president_scholarship_candidates_higher_cadet_listener,
            "government_scholarship_candidates_higher_cadet_listener": government_scholarship_candidates_higher_cadet_listener,
            "government_scholarship_candidates_secondary_cadet": ScholarshipListOptions(
                lname
            ).get_scholarship_candidates_list(
                lname, xp_key, 1, "Правительства РФ", 1, (3,)
            ),
        }
    else:
        return jsonify(success=False, message="Incorrect document type passed"), 400
    return jsonify(result)
